{
    "$graph": [
        {
            "class": "CommandLineTool",
            "id": "#ApplyBQSR.yaml",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 3500
                }
            ],
            "baseCommand": [
                "/usr/gitc/gatk4/gatk-launch",
                "ApplyBQSR"
            ],
            "arguments": [
                {
                    "position": 0,
                    "shellQuote": false,
                    "valueFrom": "--javaOptions \"-XX:+PrintFlagsFinal -XX:+PrintGCTimeStamps -XX:+PrintGCDateStamps -XX:+PrintGCDetails -Xloggc:gc_log.log -XX:GCTimeLimit=50 -XX:GCHeapFreeLimit=10 -Dsamjdk.compression_level=$(inputs.compression_level) -Xms3000m\" --createOutputBamMD5 --addOutputSAMProgramRecord -R $(inputs.ref_fasta.path) -I $(inputs.input_bam.path) --useOriginalQualities -O $(inputs.output_bam) -bqsr $(inputs.recalibration_report.path) -SQQ 10 -SQQ 20 -SQQ 30"
                }
            ],
            "inputs": [
                {
                    "id": "#ApplyBQSR.yaml/gatk4_applybqsr/input_bam",
                    "type": "File",
                    "secondaryFiles": [
                        "^.bai"
                    ]
                },
                {
                    "id": "#ApplyBQSR.yaml/gatk4_applybqsr/output_bam",
                    "type": "string"
                },
                {
                    "id": "#ApplyBQSR.yaml/gatk4_applybqsr/recalibration_report",
                    "type": "File"
                },
                {
                    "id": "#ApplyBQSR.yaml/gatk4_applybqsr/sequence_group_interval",
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-L",
                            "separate": true
                        }
                    }
                },
                {
                    "id": "#ApplyBQSR.yaml/gatk4_applybqsr/ref_fasta",
                    "type": "File",
                    "secondaryFiles": [
                        ".fai",
                        "^.dict"
                    ]
                },
                {
                    "id": "#ApplyBQSR.yaml/gatk4_applybqsr/compression_level",
                    "type": "int"
                }
            ],
            "outputs": [
                {
                    "id": "#ApplyBQSR.yaml/gatk4_applybqsr/recalibrated_bam",
                    "type": "File",
                    "outputBinding": {
                        "glob": "*bam"
                    },
                    "secondaryFiles": [
                        "^.bai",
                        "^.bam.md5"
                    ]
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#BaseRecalibrator.yaml",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 6000
                }
            ],
            "baseCommand": [
                "/usr/gitc/gatk4/gatk-launch",
                "BaseRecalibrator"
            ],
            "arguments": [
                {
                    "position": 0,
                    "shellQuote": false,
                    "valueFrom": "--javaOptions \"-XX:GCTimeLimit=50 -XX:GCHeapFreeLimit=10 -XX:+PrintFlagsFinal -XX:+PrintGCTimeStamps -XX:+PrintGCDateStamps -XX:+PrintGCDetails -Xloggc:gc_log.log -Xms4000m\" -R $(inputs.ref_fasta.path) -I $(inputs.input_bam.path) --useOriginalQualities -O $(inputs.input_bam.nameroot).recal_data.csv"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-knownSites"
                    },
                    "secondaryFiles": [
                        "^.gz.tbi"
                    ],
                    "id": "#BaseRecalibrator.yaml/baserecalibrator/dbSNP_vcf"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#BaseRecalibrator.yaml/baserecalibrator/input_bam"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File",
                        "inputBinding": {
                            "prefix": "-knownSites"
                        }
                    },
                    "secondaryFiles": [
                        "^.gz.tbi"
                    ],
                    "id": "#BaseRecalibrator.yaml/baserecalibrator/known_indels_sites_VCFs"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        "^.dict",
                        ".fai"
                    ],
                    "id": "#BaseRecalibrator.yaml/baserecalibrator/ref_fasta"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-L",
                            "separate": true
                        }
                    },
                    "id": "#BaseRecalibrator.yaml/baserecalibrator/sequence_group_interval"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.recal_data.csv"
                    },
                    "id": "#BaseRecalibrator.yaml/baserecalibrator/recalibration_report"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#ConvertToCram.yaml",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 3500
                }
            ],
            "baseCommand": [
                "samtools",
                "view"
            ],
            "arguments": [
                {
                    "position": 1,
                    "shellQuote": false,
                    "valueFrom": "-C -T $(inputs.reference.path) -o $(inputs.input_bam.nameroot).cram $(inputs.input_bam.path) && samtools index $(inputs.input_bam.nameroot).cram"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "secondaryFiles": [
                        "^.bai"
                    ],
                    "id": "#ConvertToCram.yaml/converttocram/input_bam"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".fai"
                    ],
                    "id": "#ConvertToCram.yaml/converttocram/reference"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.cram"
                    },
                    "secondaryFiles": [
                        ".crai"
                    ],
                    "id": "#ConvertToCram.yaml/converttocram/output"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#CreateSequenceGroupingTSV.yaml",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                },
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "baseCommand": [
                "python"
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#CreateSequenceGroupingTSV.yaml/createinterval/ref_dict"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 0
                    },
                    "id": "#CreateSequenceGroupingTSV.yaml/createinterval/script"
                }
            ],
            "outputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputBinding": {
                        "glob": "*.txt"
                    },
                    "id": "#CreateSequenceGroupingTSV.yaml/createinterval/groups"
                }
            ]
        },
        {
            "class": "ExpressionTool",
            "id": "#Expression_createsequencegrouping.yaml",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File",
                        "inputBinding": {
                            "loadContents": true,
                            "valueFrom": "$(null)"
                        }
                    },
                    "id": "#Expression_createsequencegrouping.yaml/expression_createsequencegrouping/sequence_grouping_with_unmapped_tsv"
                }
            ],
            "outputs": [
                {
                    "type": {
                        "type": "array",
                        "items": {
                            "type": "array",
                            "items": "string"
                        }
                    },
                    "id": "#Expression_createsequencegrouping.yaml/expression_createsequencegrouping/sequence_grouping_array"
                }
            ],
            "expression": "${\n    var outer_array = [];\n    for (var i=0; i< inputs.sequence_grouping_with_unmapped_tsv.length; i++) {\n      var inner_array = [];\n      var arrayofstring = inputs.sequence_grouping_with_unmapped_tsv[i].contents.split(\"\\t\");\n      for (var j=0; j < arrayofstring.length; j++) {\n        var sequence_contents = arrayofstring[j];\n        inner_array.push(sequence_contents);\n    }\n      outer_array.push(inner_array);\n   }\n    return {\"sequence_grouping_array\": outer_array};\n}\n"
        },
        {
            "class": "CommandLineTool",
            "id": "#GatherBamFiles.yaml",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 3500
                }
            ],
            "baseCommand": [
                "java",
                "-Dsamjdk.compression_level=2",
                "-Xms2000m",
                "-jar",
                "/usr/gitc/picard.jar"
            ],
            "arguments": [
                {
                    "position": 0,
                    "shellQuote": false,
                    "valueFrom": "GatherBamFiles OUTPUT=$(inputs.output_bam_name) CREATE_INDEX=false CREATE_MD5_FILE=true"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File",
                        "inputBinding": {
                            "prefix": "INPUT=",
                            "separate": false
                        }
                    },
                    "id": "#GatherBamFiles.yaml/gatk_gatherbamfiles/input_bam"
                },
                {
                    "type": "string",
                    "id": "#GatherBamFiles.yaml/gatk_gatherbamfiles/output_bam_name"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.bam"
                    },
                    "secondaryFiles": [
                        "^.bam.md5"
                    ],
                    "id": "#GatherBamFiles.yaml/gatk_gatherbamfiles/output_bam"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#GatherBqsrReports.yaml",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 3500
                }
            ],
            "baseCommand": [
                "/usr/gitc/gatk4/gatk-launch",
                "GatherBQSRReports"
            ],
            "arguments": [
                {
                    "position": 0,
                    "shellQuote": false,
                    "valueFrom": "--javaOptions \"-Xms3000m\" -O $(inputs.output_report_filename)"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File",
                        "inputBinding": {
                            "prefix": "-I",
                            "separate": true
                        }
                    },
                    "id": "#GatherBqsrReports.yaml/gatk_gatherbqsrreports/input_bqsr_reports"
                },
                {
                    "type": "string",
                    "id": "#GatherBqsrReports.yaml/gatk_gatherbqsrreports/output_report_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.csv"
                    },
                    "id": "#GatherBqsrReports.yaml/gatk_gatherbqsrreports/output_report"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#GetBwaVersion.yaml",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                }
            ],
            "inputs": [],
            "stdout": "version",
            "outputs": [
                {
                    "type": "string",
                    "outputBinding": {
                        "glob": "version",
                        "loadContents": true,
                        "outputEval": "$(self[0].contents.trim())"
                    },
                    "id": "#GetBwaVersion.yaml/getBwaversion/version"
                }
            ],
            "baseCommand": [
                "/usr/gitc/bwa"
            ],
            "arguments": [
                {
                    "valueFrom": "2>&1",
                    "position": 1,
                    "shellQuote": false
                },
                {
                    "valueFrom": "grep",
                    "position": 2,
                    "prefix": "|"
                },
                {
                    "valueFrom": "^Version",
                    "position": 3,
                    "prefix": "-e"
                },
                {
                    "valueFrom": "sed",
                    "position": 4,
                    "prefix": "|"
                },
                {
                    "valueFrom": "s/Version: //",
                    "position": 5
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#MarkDuplicates.yaml",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                }
            ],
            "baseCommand": [
                "java",
                "-Xms4000m",
                "-jar",
                "/usr/gitc/picard.jar",
                "MarkDuplicates"
            ],
            "arguments": [
                {
                    "position": 1,
                    "shellQuote": false,
                    "valueFrom": "OUTPUT=$(inputs.output_bam) METRICS_FILE=$(inputs.metrics_filename) VALIDATION_STRINGENCY=SILENT OPTICAL_DUPLICATE_PIXEL_DISTANCE=2500 ASSUME_SORT_ORDER=\"queryname\" CLEAR_DT=\"false\" ADD_PG_TAG_TO_READS=false READ_NAME_REGEX=$(inputs.read_name_regex)"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "prefix": "INPUT=",
                        "separate": true
                    },
                    "id": "#MarkDuplicates.yaml/markduplicates/input_bams"
                },
                {
                    "type": "string",
                    "id": "#MarkDuplicates.yaml/markduplicates/metrics_filename"
                },
                {
                    "type": "string",
                    "id": "#MarkDuplicates.yaml/markduplicates/output_bam"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "id": "#MarkDuplicates.yaml/markduplicates/read_name_regex"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.bam"
                    },
                    "id": "#MarkDuplicates.yaml/markduplicates/output_markduplicates_bam"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.txt"
                    },
                    "id": "#MarkDuplicates.yaml/markduplicates/output_markduplicates_metrics"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#SamToFastqAndBwaMemAndMba.yaml",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                },
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 14000,
                    "coresMin": 16
                }
            ],
            "baseCommand": [
                "java",
                "-Xms5000m",
                "-jar",
                "/usr/gitc/picard.jar"
            ],
            "arguments": [
                {
                    "position": 0,
                    "shellQuote": false,
                    "valueFrom": "SamToFastq INPUT=$(inputs.input_bam.path) FASTQ=/dev/stdout INTERLEAVE=true NON_PF=true | /usr/gitc/bwa mem -K 100000000 -p -v 3 -t 16 -Y $(inputs.indexed_reference_fasta.path) - 0.7.15-r1140 | java -Dsamjdk.compression_level=2 -Xms4000m -jar /usr/gitc/picard.jar MergeBamAlignment VALIDATION_STRINGENCY=SILENT EXPECTED_ORIENTATIONS=FR ATTRIBUTES_TO_RETAIN=X0 ATTRIBUTES_TO_REMOVE=NM ATTRIBUTES_TO_REMOVE=MD ALIGNED_BAM=/dev/stdin UNMAPPED_BAM=$(inputs.input_bam.path) OUTPUT=$(inputs.input_bam.nameroot).aligned.unsorted.bam REFERENCE_SEQUENCE=$(inputs.indexed_reference_fasta.path) PAIRED_RUN=true SORT_ORDER='unsorted' IS_BISULFITE_SEQUENCE=false ALIGNED_READS_ONLY=false CLIP_ADAPTERS=false MAX_RECORDS_IN_RAM=2000000 ADD_MATE_CIGAR=true MAX_INSERTIONS_OR_DELETIONS=-1 PRIMARY_ALIGNMENT_STRATEGY=MostDistant PROGRAM_RECORD_ID='bwamem' PROGRAM_GROUP_NAME='bwamem' PROGRAM_GROUP_VERSION=$inputs.bwa_version PROGRAM_GROUP_COMMAND_LINE='bwa mem -K 100000000 -p -v 3 -t 16 -Y $(inputs.indexed_reference_fasta.basename)' ALIGNER_PROPER_PAIR_FLAGS=true UNMAP_CONTAMINANT_READS=true UNMAPPED_READ_STRATEGY=COPY_TO_TAG ADD_PG_TAG_TO_READS=false"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "id": "#SamToFastqAndBwaMemAndMba.yaml/bwa_mem/bwa_version"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".64.amb",
                        ".64.ann",
                        ".64.bwt",
                        ".64.pac",
                        ".64.sa",
                        ".64.alt",
                        "^.dict"
                    ],
                    "id": "#SamToFastqAndBwaMemAndMba.yaml/bwa_mem/indexed_reference_fasta"
                },
                {
                    "type": "File",
                    "id": "#SamToFastqAndBwaMemAndMba.yaml/bwa_mem/input_bam"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.bam"
                    },
                    "id": "#SamToFastqAndBwaMemAndMba.yaml/bwa_mem/output"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#SortSam.yaml",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "ramMin": 5000
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                }
            ],
            "baseCommand": [
                "java",
                "-Dsamjdk.compression_level=2",
                "-Xms4000m",
                "-jar",
                "/usr/gitc/picard.jar"
            ],
            "arguments": [
                {
                    "position": 0,
                    "shellQuote": false,
                    "valueFrom": "SortSam INPUT=$(inputs.input_bam.path) OUTPUT=$(inputs.output_bam) SORT_ORDER=\"coordinate\" CREATE_INDEX=true CREATE_MD5_FILE=true MAX_RECORDS_IN_RAM=300000 "
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#SortSam.yaml/sortsam/input_bam"
                },
                {
                    "type": "string",
                    "id": "#SortSam.yaml/sortsam/output_bam"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.bam"
                    },
                    "secondaryFiles": [
                        "^.bai",
                        ".md5"
                    ],
                    "id": "#SortSam.yaml/sortsam/output_sorted_bam"
                }
            ]
        },
        {
            "doc": "This workflow processes high-throughput sequencing data for downstream processing\n\nRequirements/expectations :\n- Human whole-genome pair-end sequencing data in unmapped BAM (uBAM) format\n- One or more read groups, one per uBAM file, all belonging to a single sample (SM)\n- Input uBAM files must additionally comply with the following requirements:\n  - filenames all have the same suffix (we use \".unmapped.bam\")\n  - files must pass validation by ValidateSamFile\n  - reads are provided in query-sorted order\n  - all reads must have an RG tag\n- Reference genome must be Hg38 with ALT contigs\n",
            "class": "Workflow",
            "id": "#alignment_workflow.cwl",
            "requirements": [
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "int",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/compression_level"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        "^.gz.tbi"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/dbSNP_vcf"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".64.amb",
                        ".64.ann",
                        ".64.bwt",
                        ".64.pac",
                        ".64.sa",
                        ".64.alt",
                        "^.dict",
                        ".fai"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/indexed_reference_fasta"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#alignment_workflow.cwl/alignment_pipeline/input_bams"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "secondaryFiles": [
                        "^.gz.tbi"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/known_indels_sites_VCFs"
                },
                {
                    "type": "string",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/output_basename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/read_name_regex"
                },
                {
                    "type": "File",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/ref_dict"
                },
                {
                    "type": "File",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/script"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#alignment_workflow.cwl/alignment_pipeline/SortBam/output_sorted_bam",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/bqsr_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#alignment_workflow.cwl/alignment_pipeline/GatherBqsrReports/output_report",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/bqsr_reports"
                },
                {
                    "type": "File",
                    "outputSource": "#alignment_workflow.cwl/alignment_pipeline/ConvertToCram/output",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/cramfile"
                },
                {
                    "type": "File",
                    "outputSource": "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/output_markduplicates_bam",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/duplicates_marked_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/output_markduplicates_metrics",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/duplicates_marked_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#alignment_workflow.cwl/alignment_pipeline/SortSam/output_sorted_bam",
                    "id": "#alignment_workflow.cwl/alignment_pipeline/sorted_bam"
                }
            ],
            "steps": [
                {
                    "run": "#ApplyBQSR.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/compression_level",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/compression_level"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/SortSam/output_sorted_bam",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/input_bam"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/output_basename",
                            "valueFrom": "$(self + \"_Bqsr.seg.bam\")",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/output_bam"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/GatherBqsrReports/output_report",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/recalibration_report"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/indexed_reference_fasta",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/ref_fasta"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/Expression_createsequencegrouping/sequence_grouping_array",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/sequence_group_interval"
                        }
                    ],
                    "scatter": [
                        "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/sequence_group_interval"
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/recalibrated_bam"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR"
                },
                {
                    "run": "#BaseRecalibrator.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/dbSNP_vcf",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator/dbSNP_vcf"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/SortSam/output_sorted_bam",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator/input_bam"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/known_indels_sites_VCFs",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator/known_indels_sites_VCFs"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/indexed_reference_fasta",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator/ref_fasta"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/Expression_createsequencegrouping/sequence_grouping_array",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator/sequence_group_interval"
                        }
                    ],
                    "scatter": [
                        "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator/sequence_group_interval"
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator/recalibration_report"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator"
                },
                {
                    "run": "#ConvertToCram.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/SortBam/output_sorted_bam",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/ConvertToCram/input_bam"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/indexed_reference_fasta",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/ConvertToCram/reference"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/ConvertToCram/output"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/ConvertToCram"
                },
                {
                    "run": "#CreateSequenceGroupingTSV.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/ref_dict",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/CreateSequenceGroupingTSV/ref_dict"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/script",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/CreateSequenceGroupingTSV/script"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/CreateSequenceGroupingTSV/groups"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/CreateSequenceGroupingTSV"
                },
                {
                    "run": "#Expression_createsequencegrouping.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/CreateSequenceGroupingTSV/groups",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/Expression_createsequencegrouping/sequence_grouping_with_unmapped_tsv"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/Expression_createsequencegrouping/sequence_grouping_array"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/Expression_createsequencegrouping"
                },
                {
                    "run": "#GatherBamFiles.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/compression_level",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/GatherBamFiles/compression_level"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/ApplyBQSR/recalibrated_bam",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/GatherBamFiles/input_bam"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/output_basename",
                            "valueFrom": "$(self + \"_Bqsr.bam\")",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/GatherBamFiles/output_bam_name"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/GatherBamFiles/output_bam"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/GatherBamFiles"
                },
                {
                    "run": "#GatherBqsrReports.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/BaseRecalibrator/recalibration_report",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/GatherBqsrReports/input_bqsr_reports"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/output_basename",
                            "valueFrom": "$(self + \"BqsrReports.csv\")",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/GatherBqsrReports/output_report_filename"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/GatherBqsrReports/output_report"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/GatherBqsrReports"
                },
                {
                    "run": "#GetBwaVersion.yaml",
                    "in": [],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/GetBwaVersion/version"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/GetBwaVersion"
                },
                {
                    "run": "#MarkDuplicates.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/SamToFastqAndBwaMemAndMba/output",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/input_bams"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/output_basename",
                            "valueFrom": "$(self + \"_markdup.txt\")",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/metrics_filename"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/output_basename",
                            "valueFrom": "$(self + \"_markdup.bam\")",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/output_bam"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/read_name_regex",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/read_name_regex"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/output_markduplicates_bam",
                        "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/output_markduplicates_metrics"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates"
                },
                {
                    "run": "#SamToFastqAndBwaMemAndMba.yaml",
                    "scatter": [
                        "#alignment_workflow.cwl/alignment_pipeline/SamToFastqAndBwaMemAndMba/input_bam"
                    ],
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/GetBwaVersion/version",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/SamToFastqAndBwaMemAndMba/bwa_version"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/indexed_reference_fasta",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/SamToFastqAndBwaMemAndMba/indexed_reference_fasta"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/input_bams",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/SamToFastqAndBwaMemAndMba/input_bam"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/SamToFastqAndBwaMemAndMba/output"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/SamToFastqAndBwaMemAndMba"
                },
                {
                    "run": "#SortSam.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/GatherBamFiles/output_bam",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/SortBam/input_bam"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/output_basename",
                            "valueFrom": "$(self + \"_BQSR.sort.bam\")",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/SortBam/output_bam"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/SortBam/output_sorted_bam"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/SortBam"
                },
                {
                    "run": "#SortSam.yaml",
                    "in": [
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/MarkDuplicates/output_markduplicates_bam",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/SortSam/input_bam"
                        },
                        {
                            "source": "#alignment_workflow.cwl/alignment_pipeline/output_basename",
                            "valueFrom": "$(self + \"_markdup_sort.bam\")",
                            "id": "#alignment_workflow.cwl/alignment_pipeline/SortSam/output_bam"
                        }
                    ],
                    "out": [
                        "#alignment_workflow.cwl/alignment_pipeline/SortSam/output_sorted_bam"
                    ],
                    "id": "#alignment_workflow.cwl/alignment_pipeline/SortSam"
                }
            ],
            "http://purl.org/dc/terms/creator": {
                "foaf:mbox": "mailto:yilinxu@uchicago.edu",
                "foaf:name": "Yilin Xu"
            }
        },
        {
            "doc": "This workflow processes high-throughput sequencing data for downstream processing\n\nRequirements/expectations :\n- Human whole-genome pair-end sequencing data in unmapped BAM (uBAM) format\n- One or more read groups, one per uBAM file, all belonging to a single sample (SM)\n- Input uBAM files must additionally comply with the following requirements:\n  - filenames all have the same suffix (we use \".unmapped.bam\")\n  - files must pass validation by ValidateSamFile\n  - reads are provided in query-sorted order\n  - all reads must have an RG tag\n- Reference genome must be Hg38 with ALT contigs\n",
            "class": "Workflow",
            "id": "#main",
            "requirements": [
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "int",
                    "id": "#main/compression_level"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        "^.gz.tbi"
                    ],
                    "id": "#main/dbSNP_vcf"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".64.amb",
                        ".64.ann",
                        ".64.bwt",
                        ".64.pac",
                        ".64.sa",
                        ".64.alt",
                        "^.dict",
                        ".fai"
                    ],
                    "id": "#main/indexed_reference_fasta"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/input_bams"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "secondaryFiles": [
                        "^.gz.tbi"
                    ],
                    "id": "#main/known_indels_sites_VCFs"
                },
                {
                    "type": "string",
                    "id": "#main/output_basename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "id": "#main/read_name_regex"
                },
                {
                    "type": "File",
                    "id": "#main/ref_dict"
                },
                {
                    "type": "File",
                    "id": "#main/refcram"
                },
                {
                    "type": "File",
                    "id": "#main/script"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#main/alignment_pipeline/bqsr_bam",
                    "id": "#main/bqsr_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/alignment_pipeline/bqsr_reports",
                    "id": "#main/bqsr_reports"
                },
                {
                    "type": "int",
                    "outputSource": "#main/checker_md5/check",
                    "id": "#main/check"
                },
                {
                    "type": "File",
                    "outputSource": "#main/alignment_pipeline/cramfile",
                    "id": "#main/cramfile"
                },
                {
                    "type": "File",
                    "outputSource": "#main/alignment_pipeline/duplicates_marked_bam",
                    "id": "#main/duplicates_marked_bam"
                },
                {
                    "type": "File",
                    "outputSource": "#main/alignment_pipeline/duplicates_marked_metrics",
                    "id": "#main/duplicates_marked_metrics"
                },
                {
                    "type": "File",
                    "outputSource": "#main/alignment_pipeline/sorted_bam",
                    "id": "#main/sorted_bam"
                }
            ],
            "steps": [
                {
                    "run": "#alignment_workflow.cwl",
                    "in": [
                        {
                            "source": "#main/compression_level",
                            "id": "#main/alignment_pipeline/compression_level"
                        },
                        {
                            "source": "#main/dbSNP_vcf",
                            "id": "#main/alignment_pipeline/dbSNP_vcf"
                        },
                        {
                            "source": "#main/indexed_reference_fasta",
                            "id": "#main/alignment_pipeline/indexed_reference_fasta"
                        },
                        {
                            "source": "#main/input_bams",
                            "id": "#main/alignment_pipeline/input_bams"
                        },
                        {
                            "source": "#main/known_indels_sites_VCFs",
                            "id": "#main/alignment_pipeline/known_indels_sites_VCFs"
                        },
                        {
                            "source": "#main/output_basename",
                            "id": "#main/alignment_pipeline/output_basename"
                        },
                        {
                            "source": "#main/read_name_regex",
                            "id": "#main/alignment_pipeline/read_name_regex"
                        },
                        {
                            "source": "#main/ref_dict",
                            "id": "#main/alignment_pipeline/ref_dict"
                        },
                        {
                            "source": "#main/script",
                            "id": "#main/alignment_pipeline/script"
                        }
                    ],
                    "out": [
                        "#main/alignment_pipeline/duplicates_marked_bam",
                        "#main/alignment_pipeline/duplicates_marked_metrics",
                        "#main/alignment_pipeline/sorted_bam",
                        "#main/alignment_pipeline/bqsr_reports",
                        "#main/alignment_pipeline/bqsr_bam",
                        "#main/alignment_pipeline/cramfile"
                    ],
                    "id": "#main/alignment_pipeline"
                },
                {
                    "run": "#checker_md5.yaml.cwl",
                    "in": [
                        {
                            "source": "#main/alignment_pipeline/cramfile",
                            "id": "#main/checker_md5/cram"
                        },
                        {
                            "source": "#main/refcram",
                            "id": "#main/checker_md5/refcram"
                        },
                        {
                            "source": "#main/indexed_reference_fasta",
                            "id": "#main/checker_md5/reference"
                        }
                    ],
                    "out": [
                        "#main/checker_md5/check"
                    ],
                    "id": "#main/checker_md5"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "id": "#checker_md5.yaml.cwl",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "DockerRequirement",
                    "dockerPull": "us.gcr.io/broad-gotc-prod/genomes-in-the-cloud:2.3.2-1510681135"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "baseCommand": [],
            "arguments": [
                {
                    "position": 0,
                    "shellQuote": false,
                    "valueFrom": "samtools view $(inputs.refcram.path) -T $(inputs.reference.path)| md5sum > sum_file.txt &&\nsamtools view $(inputs.cram.path) -T $(inputs.reference.path)| md5sum --check sum_file.txt\n"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#checker_md5.yaml.cwl/checkcram_md5/cram"
                },
                {
                    "type": "File",
                    "id": "#checker_md5.yaml.cwl/checkcram_md5/refcram"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".fai"
                    ],
                    "id": "#checker_md5.yaml.cwl/checkcram_md5/reference"
                }
            ],
            "stdout": "check",
            "outputs": [
                {
                    "type": "int",
                    "outputBinding": {
                        "glob": "check",
                        "loadContents": true,
                        "outputEval": "${\n  var message = \"The output cram file is not identical\"\n  var flag = 1\n  if (self[0].contents.trim() == \"-: OK\"){\n    message = \"The output cram file is identical\" \n    flag = 0 \n    }\n  return flag\n }\n"
                    },
                    "id": "#checker_md5.yaml.cwl/checkcram_md5/check"
                }
            ]
        }
    ],
    "cwlVersion": "v1.0"
}
